<?php

/**
 * @file
 * Contains \Drupal\chessboard_flexible\Element\ChessboardFlexibleDiagram.
 */

namespace Drupal\chessboard_flexible\Element;

use Drupal\Core\Render\Element\RenderElement;

/**
 * @RenderElement("chessboard_flexible_diagram")
 */
class ChessboardFlexibleDiagram extends RenderElement {

  public function getInfo() {
    $image_directory_path = drupal_get_path('module', 'chessboard_flexible') . '/default';
    $dimensions = array(
      'square' => array(
        'width' => 40,
        'height' => 40,
      ),
      'border-h' => array(
        'width' => 40,
        'height' => 4,
      ),
      'border-v' => array(
        'width' => 4,
        'height' => 40,
      ),
      'border-c' => array(
        'width' => 4,
        'height' => 4,
      ),
    );
    $attributes = array(
      'square' => array(
        'class' => 'chessboard-square',
      ),
      'border-h' => array(
        'class' => 'chessboard-border-h',
      ),
      'border-v' => array(
        'class' => 'chessboard-border-v',
      ),
      'border-c' => array(
        'class' => 'chessboard-border-c',
      ),
    );
    $class = get_class($this);
    $type = array(
      '#dimensions' => $dimensions,
      '#image_directory_path' => $image_directory_path,
      '#item' => NULL,
      '#pre_render' => array(
        array($class, 'preRenderChessboardDiagram'),
      ),
      '#theme' => 'chessboard_flexible',
    );
    // Construct the translation table
    $size_attributes = $dimensions;

    // Empty and marked squares
    $table_tiles = array(
      '-0' => array('#theme' => 'image', '#uri' => $image_directory_path . '/' . chessboard_flexible_piece_filename('-', 0), '#width' => $size_attributes['square']['width'], '#height' => $size_attributes['square']['height'], '#alt' => '-', '#title' => '', '#attributes' => $attributes['square']),
      '-1' => array('#theme' => 'image', '#uri' => $image_directory_path . '/' . chessboard_flexible_piece_filename('-', 1), '#width' => $size_attributes['square']['width'], '#height' => $size_attributes['square']['height'], '#alt' => '-', '#title' => '', '#attributes' => $attributes['square']),
      'x0' => array('#theme' => 'image', '#uri' => $image_directory_path . '/' . chessboard_flexible_piece_filename('x', 0), '#width' => $size_attributes['square']['width'], '#height' => $size_attributes['square']['height'], '#alt' => 'x', '#title' => '', '#attributes' => $attributes['square']),
      'x1' => array('#theme' => 'image', '#uri' => $image_directory_path . '/' . chessboard_flexible_piece_filename('x', 1), '#width' => $size_attributes['square']['width'], '#height' => $size_attributes['square']['height'], '#alt' => 'x', '#title' => '', '#attributes' => $attributes['square']),
    );

    // Pieces
    $pieces = 'kqbnrpac';
    for ($i=0; $i<8; $i++) {
      $piece = $pieces[$i];
      $table_tiles += array(
        strtoupper($piece) . '0' => array('#theme' => 'image', '#uri' => $image_directory_path . '/' . chessboard_flexible_piece_filename(strtoupper($piece), 0), '#width' => $size_attributes['square']['width'], '#height' => $size_attributes['square']['height'], '#alt' => strtoupper($piece), '#title' => '', '#attributes' => $attributes['square']),
        strtoupper($piece) . '1' => array('#theme' => 'image', '#uri' => $image_directory_path . '/' . chessboard_flexible_piece_filename(strtoupper($piece), 1), '#width' => $size_attributes['square']['width'], '#height' => $size_attributes['square']['height'], '#alt' => strtoupper($piece), '#title' => '', '#attributes' => $attributes['square']),
        $piece . '0' => array('#theme' => 'image', '#uri' => $image_directory_path . '/' . chessboard_flexible_piece_filename($piece, 0), '#width' => $size_attributes['square']['width'], '#height' => $size_attributes['square']['height'], '#alt' => $piece, '#title' => '', '#attributes' => $attributes['square']),
        $piece . '1' => array('#theme' => 'image', '#uri' => $image_directory_path . '/' . chessboard_flexible_piece_filename($piece, 1), '#width' => $size_attributes['square']['width'], '#height' => $size_attributes['square']['height'], '#alt' => $piece, '#title' => '', '#attributes' => $attributes['square']),
      );
    }

    // Borders
    $table_tiles += array(
      'T' => array('#theme' => 'image', '#uri' => $image_directory_path . '/' . chessboard_flexible_border_filename('T'), '#width' => $size_attributes['border-h']['width'], '#height' => $size_attributes['border-h']['height'], '#title' => '', '#attributes' => $attributes['border-h']),
      'B' => array('#theme' => 'image', '#uri' => $image_directory_path . '/' . chessboard_flexible_border_filename('B'), '#width' => $size_attributes['border-h']['width'], '#height' => $size_attributes['border-h']['height'], '#title' => '', '#attributes' => $attributes['border-h']),
      'L' => array('#theme' => 'image', '#uri' => $image_directory_path . '/' . chessboard_flexible_border_filename('L'), '#width' => $size_attributes['border-v']['width'], '#height' => $size_attributes['border-v']['height'], '#title' => '', '#attributes' => $attributes['border-v']),
      'R' => array('#theme' => 'image', '#uri' => $image_directory_path . '/' . chessboard_flexible_border_filename('R'), '#width' => $size_attributes['border-v']['width'], '#height' => $size_attributes['border-v']['height'], '#title' => '', '#attributes' => $attributes['border-v']),
      'TL' => array('#theme' => 'image', '#uri' => $image_directory_path . '/' . chessboard_flexible_border_filename('TL'), '#width' => $size_attributes['border-c']['width'], '#height' => $size_attributes['border-c']['height'], '#title' => '', '#attributes' => $attributes['border-c']),
      'TR' => array('#theme' => 'image', '#uri' => $image_directory_path . '/' . chessboard_flexible_border_filename('TR'), '#width' => $size_attributes['border-c']['width'], '#height' => $size_attributes['border-c']['height'], '#title' => '', '#attributes' => $attributes['border-c']),
      'BL' => array('#theme' => 'image', '#uri' => $image_directory_path . '/' . chessboard_flexible_border_filename('BL'), '#width' => $size_attributes['border-c']['width'], '#height' => $size_attributes['border-c']['height'], '#title' => '', '#attributes' => $attributes['border-c']),
      'BR' => array('#theme' => 'image', '#uri' => $image_directory_path . '/' . chessboard_flexible_border_filename('BR'), '#width' => $size_attributes['border-c']['width'], '#height' => $size_attributes['border-c']['height'], '#title' => '', '#attributes' => $attributes['border-c']),
    );
    $type['#table_tiles'] = $table_tiles;
    return $type;
  }

  public static function preRenderChessboardDiagram($elements) {
    $elements['#file_max'] = $elements['#piece_placement']->getFileMax();
    $elements['#board'] = array();
    $size = $elements['#piece_placement']->getSize();
    for ($i = 0; $i < $size; $i++) {
      $rank_d = (int) floor($i / $elements['#file_max']);
      $file = $i - $rank_d * $elements['#file_max'];
      if ($file == 0) {
        $elements['#board'][$rank_d] = array();
      }
      $elements['#board'][$rank_d][$file] = $elements['#piece_placement']->getPieceCode($rank_d, $file);
    }
    return $elements;
  }

}
